/*
 * Java-Gnome Bindings Library
 *
 * Copyright 1998-2004 the Java-Gnome Team, all rights reserved.
 *
 * The Java-Gnome bindings library is free software distributed under
 * the terms of the GNU Library General Public License version 2.
 */

package org.gnu.gtk;

import org.gnu.glib.Type;
import org.gnu.glib.Handle;

/**
 * This widget displays a grid and traces a graph on it following a set of
 * points. There are three kinds of traces available. You can add new points,
 * and move existing points, interactively with the mouse.
 */
public class Curve extends DrawingArea {
    /**
     * Create a new Curve object.
     */
    public Curve() {
        super(gtk_curve_new());
    }

    /**
     * Construct a new Curve from a handle to a native resource.
     */
    public Curve(Handle handle) {
        super(handle);
    }

    /**
     * Internal static factory method to be used by Java-Gnome only.
     */
    public static Curve getCurve(Handle handle) {
        if (handle == null)
            return null;

        Curve obj = (Curve) getGObjectFromHandle(handle);
        if (obj == null)
            obj = new Curve(handle);

        return obj;
    }

    /**
     * Retrieve the runtime type used by the GLib library.
     */
    public static Type getType() {
        return new Type(gtk_curve_get_type());
    }

    native static final protected int gtk_curve_get_type();

    native static final protected Handle gtk_curve_new();

    native static final protected void gtk_curve_reset(Handle curve);

    native static final protected void gtk_curve_set_gamma(Handle curve,
            double gamma);

    native static final protected void gtk_curve_set_range(Handle curve,
            double minX, double maxX, double minY, double maxY);

    native static final protected void gtk_curve_get_vector(Handle curve,
            int veclen, double[] vector);

    native static final protected void gtk_curve_set_vector(Handle curve,
            int veclen, double[] vector);

    native static final protected void gtk_curve_set_curve_type(Handle curve,
            int type);

}
